[data-md-color-scheme="isahit"] {
    --md-primary-fg-color:        #FF534A;
    --md-primary-fg-color--light: #ECB7B7;
    --md-primary-fg-color--dark:  #90030C;
    --md-default-bg-color: #f8f5f1;
    --md-code-bg-color: #e7e7e7;
}

.md-header__topic .md-ellipsis{
    line-height: 2.9;
}

.md-header__topic{
    margin-left: 50px;

}

.md-typeset{
    font-size: 12pt;
}